#---------------------------------------------------------------------
#    Name: WinRegDump
# Purpose: Routines used to retrieve Windows Registry information for
#          \HKEY_LOCAL_MACHINE\Software\IBM sub-tree for WPM_PDtool
#  Author: Robert A. Gibson [rag]
# Version: 1.0
#    Note: This is in a separate file, so the "use Win32::Registry"
#          won't fail on systems (i.e., AIX) which don't have Win32.
#   Usage: require WinRegDump;
#          ...
#          WinRegDump();
#
# History:
#   date    ver  who  description
# --------  ---  ---  ----------
# 03/15/01  1.0  rag  new
#---------------------------------------------------------------------
  use Win32::Registry;                 # We depend heavily on this
#---------------------------------------------------------------------
# Dump() - Format and output binary data in hex
#
#   Parms:
#     fh      = FileHandle of output file
#     Name    = Name of item being dumped.
#     Title   = Prefix for dumped data.
#     Data    = Data to be dumped.
#---------------------------------------------------------------------
  sub Dump {
      my ( $fh, $Name, $Title, $Data ) = @_;
      print $fh "$Title";
      my $pad   = length( $Name ) + 3 + length( $Title );
      my $width = $pad;
      my $dlen  = length( $Data  );
      for ( my $i = 0; $i < $dlen; $i++ ) {
          printf $fh "%02x", substr( $Data, $i, 1 );
          $width += 2;
          if ( $i + 1 < $dlen ) {
              print $fh ",";
              if ( ++$width > 60 ) {
                  print $fh "\\\n" . " " x $pad;
                  $width = $pad;
              }
          }
      }
      print $fh "\n";
  }

#---------------------------------------------------------------------
# ProcessKey() - Recursive routine used to display registry info
#
#   Parms:
#     fh      = FileHandle of output file
#     Key     = Registry key (e.g., HKEY_LOCAL_MACHINE\Software\IBM)
#     Path    = Corresponding path to this key
#     KeyName = Subkey/subtree to be processed.
#---------------------------------------------------------------------
  sub ProcessKey {
      my ( $fh, $Key, $Path, $KeyName ) = @_;
      my $TempKey;
      if ( "" eq $KeyName ) {
          print $fh "\n[$Path]\n";
      } else {
          print $fh "\n[$Path\\$KeyName]\n";
      }
      if ( $Key -> Open( $KeyName, $TempKey ) ) {
          my ( $SubKey, $Value, @Keys, %Values );
          $Path .= "\\$KeyName" if ( "" ne $KeyName );
          if ( $TempKey -> GetValues( \%Values ) ) {
              foreach my $Value ( sort( keys( %Values ) ) ) {
                  my ( $Name, $Type, $Data ) = @{ $Values{ $Value } };
                  if ( "" eq $Value ) {
                      print $fh "@=";
                  } else {
                      print $fh "\"$Value\"=";
                  }
                  if ( REG_SZ  == $Type ) {
                      $Data =~ s/\\/\\\\/g;   # Single backslash to double
                      $Data =~ s/\x00*$//;    # Delete trailing nulls
                      print $fh "\"$Data\"\n";
                  } elsif ( REG_EXPAND_SZ == $Type ) {
                      Dump( $fh, $Name, "hex(2):", $Data );
                  } elsif ( REG_MULTI_SZ  == $Type ) {
                      Dump( $fh, $Name, "hex(7):", $Data );
                  } elsif ( REG_DWORD     == $Type ) {
                      printf $fh "dword:%08x (%d)\n", $Data, $Data;
                  } elsif ( REG_BINARY    == $Type ) {
                      Dump( $fh, $Name, "hex:", $Data );
                  } else {
                      print $fh "Unknown type: \"$Type\"\n";
                  }
              }
          }
          $TempKey -> GetKeys( \@Keys );

          foreach $SubKey ( sort( @Keys ) ) {
              ProcessKey( $fh, $TempKey,  $Path,  $SubKey );
          }
          $TempKey -> Close();
      }
  }

#---------------------------------------------------------------------
# WinRegDump() - Display the Software/IBM subtree of Windows registry
#---------------------------------------------------------------------
  sub WinRegDump {
      my $ibmsw = "ibmsw.out";         # Output file name
      print "RegDump \\HKEY_LOCAL_MACHINE\\Software\\IBM >$ibmsw";
      open( IBMSW, ">$ibmsw" ) or die "Unable to open $ibmsw: $!";
      my $Path = "SOFTWARE\\IBM";      # Sub-tree to be displayed
      $HKEY_LOCAL_MACHINE -> Open( $Path, $ibm );
      ProcessKey( *IBMSW, $ibm, "HKEY_LOCAL_MACHINE\\$Path", "" );
      $HKEY_LOCAL_MACHINE -> Close();
      close( IBMSW );                  # Flush & close file
      print "\n";                      # Dump complete
  }

1;
